'''Blog Data.

Entry, Month, Index, and Recent.
'''
#
# Copyright (c) 2006 Satoshi Fukutomi <info@fuktommy.com>.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE AUTHORS AND CONTRIBUTORS ``AS IS'' AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHORS OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
# OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
# OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
# SUCH DAMAGE.
#

import os
import re
import csv
import time

__all__ = ['Entry', 'Month', 'Index', 'recent_entry', 'set_recent_entry']

#
# Config
#
data_dir = './data'


class Entry:
    '''An Article.
    '''
    def __init__(self, id, title='', body=''):
        self.id = int(id)
        self.title = title
        self.body = body

    def __nonzero__(self):
        return self.exists()

    def month(self):
        return time.strftime('%Y-%m', time.localtime(self.id))
    
    def dirname(self):
        return os.path.join(data_dir, self.month())

    def path(self):
        return '%s/%d.txt' % (self.dirname(), self.id)

    def exists(self):
        return os.path.exists(self.path())

    def load(self):
        if os.path.isfile(self.path()):
            f = file(self.path())
            self.title = f.readline().strip()
            self.body = f.read().strip()

    def sync(self):
        if not os.path.isdir(self.dirname()):
            os.makedirs(self.dirname())
        f = file(self.path(), 'wb')
        f.write('%s\n' % self.title)
        f.write(self.body)

    def remove(self):
        os.remove(self.path())

# End of Entry


class Month:
    '''Entries in a Month.
    '''
    def __init__(self, month):
        self.month = month
        self.entries = []

    def __iter__(self):
        return iter(self.entries)

    def __len__(self):
        return len(self.entries)

    def dirname(self):
        return os.path.join(data_dir, self.month)

    def path(self):
        return os.path.join(data_dir, self.month, 'index.csv')

    def exists(self):
        return os.path.exists(self.path())

    def load(self):
        if os.path.isfile(self.path()):
            reader = csv.reader(file(self.path()))
            for row in reader:
                self.append(row)
        self.entries.sort(lambda a,b: cmp(int(b[0]), int(a[0])))

    def append(self, data):
        self.entries.append(data)

    def __len__(self):
        if not self.entries:
            self.load()
        return len(self.entries)

    def sync(self):
        if not os.path.isdir(self.dirname()):
            os.makedirs(self.dirname())
        writer = csv.writer(file(self.path(), 'wb'))
        for i in self.entries:
            writer.writerow(i)

    def remove(self):
        os.remove(self.path())
        os.rmdir(self.dirname())

    def update(self):
        self.entries = []
        for i in os.listdir(self.dirname()):
            found = re.search(r'(\d+).txt', i)
            if found:
                id = found.group(1)
                entry = Entry(id)
                entry.load()
                self.append((entry.id, entry.title))
        self.sync()

# End of Month


class Index:
    '''Months.
    '''
    def __init__(self):
        self.months = []

    def __iter__(self):
        return iter(self.months)

    def load(self):
        buf = os.listdir(data_dir)
        for i in buf:
            if re.search(r'^\d{4}-\d{2}$', i):
                self.months.append(i)
        self.months.sort()
        self.months.reverse()

    def __len__(self):
        if not self.months:
            self.load()
        return len(self.months)

# End of Index


def recent_entry():
    id = file(os.path.join(data_dir, 'new.txt')).read().strip()
    return Entry(id)

def set_recent_entry(entry):
    file(os.path.join(data_dir, 'new.txt'), 'w').write('%d' % entry.id)
